import {
  GLOBAL_SCRIPT_SETUP_ID_PREFIX
} from "./chunk-4KRXJAJT.mjs";

// src/global-script-setup/index.ts
import path from "path";
import {
  DEFINE_SETUP_COMPONENT,
  MagicString,
  babelParse,
  getTransformResult
} from "@vue-macros/common";

// ../../node_modules/.pnpm/estree-walker@3.0.1/node_modules/estree-walker/src/walker.js
var WalkerBase = class {
  constructor() {
    this.should_skip = false;
    this.should_remove = false;
    this.replacement = null;
    this.context = {
      skip: () => this.should_skip = true,
      remove: () => this.should_remove = true,
      replace: (node) => this.replacement = node
    };
  }
  replace(parent, prop, index, node) {
    if (parent) {
      if (index !== null) {
        parent[prop][index] = node;
      } else {
        parent[prop] = node;
      }
    }
  }
  remove(parent, prop, index) {
    if (parent) {
      if (index !== null) {
        parent[prop].splice(index, 1);
      } else {
        delete parent[prop];
      }
    }
  }
};

// ../../node_modules/.pnpm/estree-walker@3.0.1/node_modules/estree-walker/src/sync.js
var SyncWalker = class extends WalkerBase {
  constructor(enter, leave) {
    super();
    this.enter = enter;
    this.leave = leave;
  }
  visit(node, parent, prop, index) {
    if (node) {
      if (this.enter) {
        const _should_skip = this.should_skip;
        const _should_remove = this.should_remove;
        const _replacement = this.replacement;
        this.should_skip = false;
        this.should_remove = false;
        this.replacement = null;
        this.enter.call(this.context, node, parent, prop, index);
        if (this.replacement) {
          node = this.replacement;
          this.replace(parent, prop, index, node);
        }
        if (this.should_remove) {
          this.remove(parent, prop, index);
        }
        const skipped = this.should_skip;
        const removed = this.should_remove;
        this.should_skip = _should_skip;
        this.should_remove = _should_remove;
        this.replacement = _replacement;
        if (skipped)
          return node;
        if (removed)
          return null;
      }
      for (const key in node) {
        const value = node[key];
        if (typeof value !== "object") {
          continue;
        } else if (Array.isArray(value)) {
          for (let i = 0; i < value.length; i += 1) {
            if (value[i] !== null && typeof value[i].type === "string") {
              if (!this.visit(value[i], node, key, i)) {
                i--;
              }
            }
          }
        } else if (value !== null && typeof value.type === "string") {
          this.visit(value, node, key, null);
        }
      }
      if (this.leave) {
        const _replacement = this.replacement;
        const _should_remove = this.should_remove;
        this.replacement = null;
        this.should_remove = false;
        this.leave.call(this.context, node, parent, prop, index);
        if (this.replacement) {
          node = this.replacement;
          this.replace(parent, prop, index, node);
        }
        if (this.should_remove) {
          this.remove(parent, prop, index);
        }
        const removed = this.should_remove;
        this.replacement = _replacement;
        this.should_remove = _should_remove;
        if (removed)
          return null;
      }
    }
    return node;
  }
};

// ../../node_modules/.pnpm/estree-walker@3.0.1/node_modules/estree-walker/src/index.js
function walk(ast, { enter, leave }) {
  const instance = new SyncWalker(enter, leave);
  return instance.visit(ast, null);
}

// src/global-script-setup/index.ts
import { normalizePath } from "@rollup/pluginutils";
var transformGlobalScriptSetup = (code, id, ctx) => {
  const normalizedId = normalizePath(id);
  const program = babelParse(code, path.extname(id).replace(/^\./, ""));
  const nodes = [];
  walk(program, {
    enter(node) {
      if (node.type === "CallExpression" && node.callee.type === "Identifier" && node.callee.name === DEFINE_SETUP_COMPONENT) {
        nodes.push(node);
      }
    }
  });
  if (nodes.length === 0)
    return;
  const s = new MagicString(code);
  const nodeContexts = nodes.map((node, i) => {
    if (!["FunctionExpression", "ArrowFunctionExpression"].includes(
      node.arguments[0].type
    ))
      throw new SyntaxError(
        `${DEFINE_SETUP_COMPONENT}: Invalid setup component definition`
      );
    const importName = `setupComponent_${i}`;
    s.overwrite(node.start, node.end, importName);
    s.prepend(
      `import ${importName} from '${GLOBAL_SCRIPT_SETUP_ID_PREFIX}${normalizedId}/${i}.vue'
`
    );
    const body = node.arguments[0].body;
    let bodyStart = body.start;
    let bodyEnd = body.end;
    if (body.type === "BlockStatement") {
      bodyStart++;
      bodyEnd--;
    }
    return {
      code: code.slice(node.start, node.end),
      body: code.slice(bodyStart, bodyEnd),
      node
    };
  });
  ctx[normalizedId] = nodeContexts;
  return getTransformResult(s, id);
};
var loadGlobalScriptSetup = (id, ctx) => {
  var _a;
  const { dir: file, name: i } = path.parse(
    id.replace(GLOBAL_SCRIPT_SETUP_ID_PREFIX, "")
  );
  const index = +i;
  if (!((_a = ctx[file]) == null ? void 0 : _a[index]))
    return;
  const { body } = ctx[file][index];
  const lang = path.extname(file).replace(/^\./, "");
  const code = `<script setup${lang ? ` lang="${lang}"` : ""}>
${body}
<\/script>`;
  return code;
};

export {
  transformGlobalScriptSetup,
  loadGlobalScriptSetup
};
